/* global Modal, $, errorMsg, successMsg, ajax */
/* eslint-disable func-names, prefer-arrow-callback, object-shorthand, prefer-template, prefer-destructuring */
/* eslint-env jquery, browser */

$(function () {
  function initCredentialsForm(form) {
    const resetButton = form.find('a#reset');

    function empty(value) {
      return !value || value.trim().length === 0;
    }

    const requireUsernamePasswordMessage =
      'You must supply both a username and password.';
    const requireUniqueUsernamesMessage = 'Usernames must be unique.';
    const passwordMeetsPolicyMessage = 'Password is not complex enough.';

    const passwordMeetsPolicy = function (value, element) {
      if (empty(value)) {
        return true;
      }

      return value.length >= 8
        && /\d/.test(value) // contains at least one digit
        && !/^[a-z0-9]+$/i.test(value); // contains at least one special character (not digit or letter)
    }

    const hasRequiredPassword = function (value, element) {
      if (!empty(value)) {
        return true;
      }

      const row = $(element).parents('tr.custom-row');

      const username = row.find('.username input').val();
      if (empty(username)) {
        return false;
      }

      const originalUsername = row.attr('data-original-username');
      if (empty(originalUsername)) {
        return false; // new user, password required
      }

      return username === originalUsername;
    };

    $.validator.addMethod(
      'maybeRequirePassword',
      hasRequiredPassword,
      requireUsernamePasswordMessage
    );

    $.validator.addMethod(
      'passwordMeetsPolicy',
      passwordMeetsPolicy,
      passwordMeetsPolicyMessage
    );

    $.validator.addMethod(
      'requireUsername',
      function (value) {
        return value && value.trim().length > 0;
      },
      requireUsernamePasswordMessage
    );

    $.validator.addMethod(
      'requireUniqueUsername',
      function (value, element) {
        const anyRequirePassword =
          form.find('.password input').filter(function () {
            return !hasRequiredPassword($(this).val(), this);
          }).length > 0;

        if (anyRequirePassword) {
          return true;
        }

        const anyEmptyUsernames =
          form.find('.username input').filter(function () {
            return empty($(this).val());
          }).length > 0;

        if (anyEmptyUsernames) {
          return true;
        }

        return (
          form.find('.username input').filter(function () {
            return this !== element && $(this).val() === value;
          }).length === 0
        );
      },
      requireUniqueUsernamesMessage
    );

    const rowTemplate =
      '<tr class=\'custom-row\'>\n' +
      '  <td class=\'username\'>\n' +
      '    <input type=\'text\'/>\n' +
      '  </td>\n' +
      '  <td class=\'password\'>\n' +
      '    <input type=\'password\'/>\n' +
      '  </td>\n' +
      '  <td class=\'level\'>\n' +
      '    <select>\n' +
      '      <option value=\'READ\' selected>Read</option>\n' +
      '      <option value=\'READWRITE\'>Read & write</option>\n' +
      '    </select>\n' +
      '  </td>\n' +
      '  <td class=\'note\'>\n' +
      '    <input type=\'text\' placeholder=\'(notes)\'/>\n' +
      '  </td>\n' +
      '  <td class=\'delete\'>\n' +
      '    <a href=\'#\'>Delete</a>\n' +
      '  </td>\n' +
      '</tr>\n';

    const hasAttribute = function (elem, attr) {
      const attribute = $(elem).attr(attr);
      return typeof attribute !== typeof undefined && attribute !== false;
    };

    function getFormState() {
        return {
          state: form.serialize(),
          html: form.find('table tbody').html()
        };
    }

    const syncResetButtonState = function () {
        if (form.state.state !== getFormState().state) {
          resetButton.removeAttr('disabled');
        } else {
          resetButton.attr('disabled', 'yes');
        }
    };

    const resetPasswordPlaceholder = function () {
      const row = $(this).parents('.custom-row');
      const username = row.find('.username input').val();
      const originalUsername = row.attr('data-original-username');
      if (!empty(originalUsername) && originalUsername === username) {
        $(this).attr('placeholder', '(hidden)');
      } else {
        $(this).removeAttr('placeholder');
      }
    };

    const syncPasswordAndResetButtonState = function () {
      syncResetButtonState(this);
      resetPasswordPlaceholder.call(
        $(this)
          .parents('tr.custom-row')
          .find('input[name^=password]')
      );
    };

    const initOriginalUsernameState = function () {
      const customRows = form.find('tr.custom-row');
      customRows.each(function () {
        $(this).attr(
          'data-original-username',
          $(this)
            .find('.username input')
            .val()
        );
      });
    };

    const initFormBehavior = function () {
      const customRows = form.find('tr.custom-row');
      form.find('#anonymousLevel')
        .off('change')
        .on('change', syncResetButtonState);
      if (customRows.length !== 0) {
        customRows.each(function (rowIndex) {
          $(this)
            .find('.username input')
            .attr('name', 'username[' + rowIndex + ']')
            .off('keyup')
            .on('keyup', syncPasswordAndResetButtonState)
            .rules('add', {
              requireUsername: true,
              requireUniqueUsername: true
            });
          $(this)
            .find('.password input')
            .attr('name', 'password[' + rowIndex + ']')
            .off('keyup')
            .on('keyup', syncResetButtonState)
            .attr('placeholder', resetPasswordPlaceholder)
            .rules('add', {
              maybeRequirePassword: true,
              passwordMeetsPolicy: true
            });
          $(this)
            .find('.level select')
            .attr('name', 'level[' + rowIndex + ']')
            .off('change')
            .on('change', syncResetButtonState);
          $(this)
            .find('.note input')
            .attr('name', 'note[' + rowIndex + ']')
            .off('keyup')
            .on('keyup', syncResetButtonState);
          $(this)
            .find('.delete a')
            .off('click')
            .on('click', function () {
              const button = $(this);
              Modal.open(
                'Delete user?',
                'This operation cannot be undone.',
                [{ value: 'Delete', classname: 'destructive' }, { value: 'Cancel' }],
                (result) => {
                  if (result === 'Delete') {
                    button.parents('tr').remove();
                    initFormBehavior();
                    syncResetButtonState();
                  }
                }
              );
              return false;
            });
        });
      }
    };

    form.find('#add-cred').on('click', function () {
      form.find('table tbody').append(rowTemplate);
      initFormBehavior();
      syncResetButtonState();
      form.find('.username:last input').focus();
      return false;
    });

    resetButton.on('click', function () {
      if (!hasAttribute($(this), 'disabled')) {
        form.find('table tbody').html(form.state.html);
        errorMsg('');
        initFormBehavior();
        syncResetButtonState();
      }
      return false;
    });

    form.validate({
      ignore: '',
      onfocusout: false,
      onkeyup: false,
      errorPlacement: function () {
      },
      showErrors: function (errorMap, errorList) {
        const arrayLength = errorList.length;
        for (let errorIndex = 0; errorIndex < arrayLength; errorIndex++) {
          if (errorIndex === 0) {
            errorMsg(errorList[errorIndex].message, form.find('div.msg'));
          }
          $(errorList[errorIndex].element).addClass('error');
        }
        this.defaultShowErrors();
      },
      highlight: function (element, errorClass) {
        $(element).addClass(errorClass);
      },
      unhighlight: function (element, errorClass) {
        $(element).removeClass(errorClass);
      },
      submitHandler: function (unused, event) {
        event.preventDefault();

          const payload = form.serialize();
          ajax(form.attr('action'), 'post', payload)
            .done(function () {
              successMsg(form.data('successMsg'), form.find('.msg'));
              InsecureWarningBanner.updateAnonymousCacheAccess(form.find('#anonymousLevel').val() === 'READWRITE');
              InaccessibleWarningBanner.updateEffectivelyInaccessibleCacheAccess(
                form.find('#anonymousLevel').val() === 'NONE'
                && form.find('.username').length === 0
              )
              form.find('.password input').val('');
              initOriginalUsernameState();

              form.find('input, select').each(function () {
                const $this = $(this);
                if ($this.is('select')) {
                  $this.find('option').each(function () {
                    const option = $(this);
                    option.attr('selected', $(this).attr('value') === $this.val() ? 'selected' : null);
                  });
                } else {
                  $this.attr('value', $this.val());
                }
              });

              form.state = getFormState();
              // eslint-disable-next-line no-use-before-define
              initFormBehavior();
              syncResetButtonState();
            })
            .fail(function (jqXHR) {
              if (jqXHR.status === 401) {
                window.location.reload(true);
              } else {
                errorMsg(form.data('failureMsg'), form.find('.msg'));
              }
            });

        return false;
      }
    });

    initOriginalUsernameState();
    initFormBehavior();
    form.state = getFormState();
    syncResetButtonState();
  }

  initCredentialsForm($('form#credentials'));
});
